#!/usr/bin/env node
'use strict';

const program = require('commander'),
  inquirer = require('inquirer'),
  chalk = require('chalk'),  
  fs = require('fs'),
  path = require('path'),
  rimraf = require('rimraf'),
  exec = require('child_process').execSync;

class TemplateFile {
  constructor(fileName, dir, variables) {
    this.fileName = fileName;
    this.dir = dir;
    this.variables = variables;
  }
}

class File {
  constructor(fileName, dir, contents) {
    this.fileName = fileName;
    this.dir = dir;
    this.contents = contents;
  }
}

let getFileTemplateContents = templateFile => {
  const templatesPath = path.join(__dirname, '/templates');
  const filePath = path.join(templatesPath, templateFile.fileName);
  let contents = fs.readFileSync(filePath).toString('utf-8');

  if (templateFile.variables) {
     contents = replaceTextVariables(templateFile.variables, contents);
  }

  return contents;
}

let generateFolders = folderName => {
  fs.mkdirSync(path.join(__dirname, folderName));
  fs.mkdirSync(path.join(__dirname, folderName, 'src'));
  fs.mkdirSync(path.join(__dirname, folderName, 'src', 'js'));
  fs.mkdirSync(path.join(__dirname, folderName, 'src', 'styles'));
}

let createFolder = async name => {
  const folderName = name; //TODO: allow names with spaces and format to valid folder name
  if (!fs.existsSync(folderName)) {
    generateFolders(folderName);
  } else {
    await inquirer.prompt([{
      type: 'confirm',
      default: false,
      name: 'overwrite',
      message: chalk.yellow(`The folder "${folderName}" already exists. Would you like to overwrite it?`)
    }]).then(({ overwrite }) => {
      if (overwrite) {
        rimraf.sync('./' + folderName);
        generateFolders(folderName);
      }
      else {
        console.log(chalk.red(`Generation Aborted. Folder "${folderName}" already exists in directory.`))
        process.exit();
      }
    })
  }
  return path.join(__dirname, folderName);
}

let writeToFolder = (dir, fileName, contents) => {
  fs.writeFileSync(`${dir}/${fileName}`, contents);
}

let replaceTextVariables = (replaceArray, contents) => {
  for (let i = 0; i < replaceArray.length; i++){
    contents = contents.split(`<<${i}>>`).join(replaceArray[i]);
  }
  return contents;
}

let generateFiles = templateFiles => {
  let files = [];
  for (let i = 0; i < templateFiles.length; i++) {
    const templateFile = templateFiles[i];
    let contents = getFileTemplateContents(templateFile);

    files.push(new File(templateFile.fileName, templateFile.dir, contents));
  }

  return files;
}

let writeFiles = (dir, files) => {
  for (let i = 0; i < files.length; i++) {
    const file = files[i];
    writeToFolder(`${dir}/${file.dir}`, file.fileName, file.contents);
  }
}

let installDeps = folderDir => {
  exec(`cd ${folderDir} && npm install`,{stdio:[0,1,2]});
}

let generate = async name => {
  const templateFiles = [
    new TemplateFile('index.php', '.'),
    new TemplateFile('style.css', '.', [name]), //TODO: allow names with spaces and format to valid text domain
    new TemplateFile('functions.php', '.', [name]), //TODO: allow names with spaces and format to valid function name
    new TemplateFile('header.php', '.'),
    new TemplateFile('footer.php', '.'),
    new TemplateFile('package.json', '.', [name.toLowerCase()]), //TODO: allow names with spaces and format to valid package name
    new TemplateFile('browserslist', '.'),
    new TemplateFile('webpack.config.js', '.', [name]),
    new TemplateFile('index.js', 'src/js'),
    new TemplateFile('index.scss', 'src/styles')
  ];

  const files = generateFiles(templateFiles);
  const folderDir = await createFolder(name);
  writeFiles(folderDir, files);

  installDeps(folderDir);
}

let main = async name => {
  console.log(chalk.blue(`Generating theme "${name}"...`));
  
  await generate(name)
    .catch((err) => {
      console.log('Oops... something went wrong :(');
      console.log(chalk.red(err));
      process.exit(1);
    });
  console.log(chalk.green(`Theme "${name}" successfully generated!`));
}

program
  .version('0.0.1')
  .command('new <name>')
  .description('Generate a new theme')
  .action(main);
program.parse(process.argv);

if (program.args.length === 0) program.help();
